using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;
using Vevo.Shared.DataAccess;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class CultureRepositoryNoCache : ICultureRepository
    {
        #region Private
        private const string AllFields = " CultureID , Name , DisplayName , Enabled ";

        public static int GetMaxCultureID()
        {
            string sql = "SELECT Max(CultureID) FROM Culture ";
            return ConvertUtilities.ToInt32( DataAccess.ExecuteScalar( sql ) );
        }

        private Culture Create( Culture culture )
        {
            int cultureID = GetMaxCultureID() + 1;

            DataAccess.ExecuteNonQuery(
                "INSERT INTO Culture (CultureID ,  Name , DisplayName , Enabled ) " +
                "VALUES (@CultureID , @Name , @DisplayName , @Enabled); ",
                DataAccess.CreateParameterInt32( cultureID ),
                DataAccess.CreateParameterString( culture.Name ),
                DataAccess.CreateParameterString( culture.DisplayName ),
                DataAccess.CreateParameterBool( culture.Enabled )
            );
            culture.CultureID = cultureID.ToString();
            return culture;
        }

        private Culture Update( Culture culture )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE Culture " +
                "SET Name = @Name , DisplayName = @DisplayName , Enabled = @Enabled " +
                "WHERE CultureID = @CultureID;",
                DataAccess.CreateParameterString( culture.Name ),
                DataAccess.CreateParameterString( culture.DisplayName ),
                DataAccess.CreateParameterBool( culture.Enabled ),
                DataAccess.CreateParameterString( culture.CultureID )
            );

            return culture;
        }

        private Culture CreateOneFromRow( DataRow row )
        {
            Culture culture = new Culture();
            culture.CultureID = row["CultureID"].ToString();
            culture.Name = row["Name"].ToString();
            culture.DisplayName = row["DisplayName"].ToString();
            culture.Enabled = ConvertUtilities.ToBoolean( row["Enabled"] );

            return culture;
        }

        private IList<Culture> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<Culture> list = new List<Culture>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        private static string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower() == "cultureid")
                return "ORDER BY CultureID ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        #endregion

        #region Public Methods

        public Culture Save( Culture culture )
        {
            Culture result;

            if (culture.CultureID == "0")
                result = Create( culture );
            else
                result = Update( culture );

            return result;
        }

        public Culture GetOne( string cultureID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM Culture " +
                "WHERE CultureID = @CultureID; ",
                DataAccess.CreateParameterString( cultureID ) );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return Culture.Null;
        }

        public string GetIDByName( string name )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CultureID " +
                "FROM Culture " +
                "WHERE Name = @Name; ",
                DataAccess.CreateParameterString( name ) );
            if (table.Rows.Count == 0)
                return "0";
            else
                return table.Rows[0]["CultureID"].ToString();
        }

        public void Delete( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM Culture " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public IList<Culture> GetAll()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CultureID, Name, DisplayName, Enabled " +
                "FROM Culture; " );

            DataColumn[] keys = new DataColumn[] { table.Columns["CultureID"] };
            table.PrimaryKey = keys;
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        public IList<Culture> GetByEnabled( bool enabled )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CultureID, Name, DisplayName, Enabled " +
                "FROM Culture " +
                "WHERE Enabled = @Enalbed; ",
                DataAccess.CreateParameterBool( enabled )
                );

            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        public IList<Culture> SearchCulture(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchFilter, searchFilter.FieldName, parameterList );

            DataTable full = DataAccess.ExecuteSelect(
                "SELECT CultureID , Name , DisplayName , Enabled " +
                "FROM  Culture " +
                commandSearch +
                CreateSortByText( sortBy ) + ";",
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            howManyItems = full.Rows.Count;

            DataColumn[] keys = new DataColumn[] { full.Columns["CultureID"] };
            full.PrimaryKey = keys;
            return ConvertTableToList( full, startIndex, endIndex );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT CultureID , Name , DisplayName , Enabled " +
                "FROM  Culture " +
                " WHERE 0 = 1; " );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
