using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Marketing;
using Vevo.Domain.Users;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class GiftRegistryRepositoryNoCache : IGiftRegistryRepository
    {
        #region Private

        private const string AllFields =
            "GiftRegistryID, EventName, EventDate, CustomerID, UserName, FirstName, LastName, Company, " +
            "Address1, Address2, City, State, Zip, Country, Phone, Fax, Residential, " +
            "HideAddress, HideEvent, NotifyNewOrder, StoreID ";

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else
                return "ORDER BY " + sortBy;
        }

        private string ConditionString( string fieldName, string fieldValue, ArrayList parameters )
        {
            if (!String.IsNullOrEmpty( fieldValue ))
            {
                parameters.Add( DataAccess.CreateParameterString(
                    "%" + fieldValue + "%" ) );

                return "AND [" + fieldName + "] LIKE @" + fieldName + " ";
            }
            else
                return "";
        }

        private string ConditionDate( string date1, string date2, ArrayList parameters )
        {
            if (string.IsNullOrEmpty( date1 ) && string.IsNullOrEmpty( date2 ))
            {
                return "";
            }
            else if (string.IsNullOrEmpty( date1 ))
            {
                parameters.Add( DataAccess.CreateParameterDateTime(
                    Convert.ToDateTime( date2, new CultureInfo( SystemConst.USCultureName ) ) ) );
                return "AND [EventDate] = @EventDate ";
            }
            else if (string.IsNullOrEmpty( date2 ))
            {
                parameters.Add( DataAccess.CreateParameterDateTime(
                    Convert.ToDateTime( date1, new CultureInfo( SystemConst.USCultureName ) ) ) );
                return "AND [EventDate] = @EventDate ";
            }
            else
            {
                parameters.Add( DataAccess.CreateParameterDateTime(
                    Convert.ToDateTime( date1, new CultureInfo( SystemConst.USCultureName ) ) ) );
                parameters.Add( DataAccess.CreateParameterDateTime(
                    Convert.ToDateTime( date2, new CultureInfo( SystemConst.USCultureName ) ) ) );
                return "AND [EventDate] Between @EventDate AND @EventDate1 ";
            }
        }

        private GiftRegistry Create( GiftRegistry giftRegistry )
        {
            string giftRegistryID = DataAccess.ExecuteScalar(
                "INSERT INTO GiftRegistry " +
                "(EventName, EventDate, CustomerID, UserName, FirstName, LastName, Company, " +
                "Address1, Address2, City, State, Zip, Country, Phone, Fax, Residential, " +
                "HideAddress, HideEvent, NotifyNewOrder, StoreID) " +
                "VALUES (@EventName, @EventDate, @CustomerID, @UserName, " +
                "@FirstName, @LastName, @Company, @Address1, " +
                "@Address2, @City, @State, @Zip, @Country, @Phone, @Fax, @Residential, " +
                "@HideAddress, @HideEvent, @NotifyNewOrder, @StoreID); " +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( giftRegistry.EventName ),
                DataAccess.CreateParameterDateTime( giftRegistry.EventDate ),
                DataAccess.CreateParameterString( giftRegistry.CustomerID ),
                DataAccess.CreateParameterString( giftRegistry.UserName ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.FirstName ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.LastName ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Company ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Address1 ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Address2 ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.City ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.State ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Zip ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Country ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Phone ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Fax ),
                DataAccess.CreateParameterBool( giftRegistry.ShippingAddress.Residential ),
                DataAccess.CreateParameterBool( giftRegistry.HideAddress ),
                DataAccess.CreateParameterBool( giftRegistry.HideEvent ),
                DataAccess.CreateParameterBool( giftRegistry.NotifyNewOrder ),
                DataAccess.CreateParameterString(giftRegistry.StoreID)
            );

            giftRegistry.GiftRegistryID = giftRegistryID;
            return giftRegistry;
        }

        private GiftRegistry Update( GiftRegistry giftRegistry )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE GiftRegistry SET " +
                "EventName = @EventName, " +
                "EventDate = @EventDate, " +
                "UserName = @UserName, " +
                "FirstName = @FirstName, " +
                "LastName = @LastName, " +
                "Company = @Company, " +
                "Address1 = @Address1, " +
                "Address2 = @Address2, " +
                "City = @City, " +
                "State = @State, " +
                "Zip = @Zip, " +
                "Country = @Country, " +
                "Phone = @Phone, " +
                "Fax = @Fax, " +
                "Residential = @Residential, " +
                "HideAddress = @HideAddress, " +
                "HideEvent = @HideEvent, " +
                "NotifyNewOrder = @NotifyNewOrder, " +
                "StoreID = @StoreID " +
                "WHERE GiftRegistryID = @GiftRegistryID;",
                DataAccess.CreateParameterString( giftRegistry.EventName ),
                DataAccess.CreateParameterDateTime( giftRegistry.EventDate ),
                DataAccess.CreateParameterString( giftRegistry.UserName ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.FirstName ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.LastName ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Company ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Address1 ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Address2 ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.City ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.State ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Zip ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Country ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Phone ),
                DataAccess.CreateParameterString( giftRegistry.ShippingAddress.Fax ),
                DataAccess.CreateParameterBool( giftRegistry.ShippingAddress.Residential ),
                DataAccess.CreateParameterBool( giftRegistry.HideAddress ),
                DataAccess.CreateParameterBool( giftRegistry.HideEvent ),
                DataAccess.CreateParameterBool( giftRegistry.NotifyNewOrder ),
                DataAccess.CreateParameterString(giftRegistry.StoreID),
                DataAccess.CreateParameterString( giftRegistry.GiftRegistryID )
            );

            return giftRegistry;
        }

        private GiftRegistry CreateOneFromRow( DataRow row )
        {
            GiftRegistry giftRegistry = new GiftRegistry();

            giftRegistry.GiftRegistryID = row["GiftRegistryID"].ToString();
            giftRegistry.EventName = row["EventName"].ToString();
            giftRegistry.EventDate = Convert.ToDateTime( row["EventDate"] );
            giftRegistry.CustomerID = row["CustomerID"].ToString();
            giftRegistry.UserName = row["Username"].ToString();
            giftRegistry.ShippingAddress = new ShippingAddress( new Address(
                row["FirstName"].ToString(), row["LastName"].ToString(),
                row["Company"].ToString(), row["Address1"].ToString(),
                row["Address2"].ToString(), row["City"].ToString(),
                row["State"].ToString(), row["Zip"].ToString(),
                row["Country"].ToString(), row["Phone"].ToString(),
                row["Fax"].ToString() ),
                ConvertUtilities.ToBoolean( row["Residential"] ) );
            giftRegistry.HideAddress = ConvertUtilities.ToBoolean( row["HideAddress"] );
            giftRegistry.HideEvent = ConvertUtilities.ToBoolean( row["HideEvent"] );
            giftRegistry.NotifyNewOrder = ConvertUtilities.ToBoolean( row["NotifyNewOrder"] );
            giftRegistry.StoreID = row["StoreID"].ToString();
            return giftRegistry;
        }

        private IList<GiftRegistry> ConvertTableToList( DataTable table )
        {
            return ConvertTableToList( table, 0, table.Rows.Count );
        }

        private IList<GiftRegistry> ConvertTableToList( DataTable table, int startIndex, int endIndex )
        {
            IList<GiftRegistry> list = new List<GiftRegistry>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( table.Rows[i] ) );

            return list;
        }

        #endregion


        #region Public Methods

        public GiftRegistry Save( GiftRegistry giftRegistry )
        {
            GiftRegistry result;

            if (giftRegistry.GiftRegistryID == "0")
                result = Create( giftRegistry );
            else
                result = Update( giftRegistry );

            return result;
        }

        public GiftRegistry GetOne( string giftRegistryID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM GiftRegistry " +
                "WHERE GiftRegistryID = @GiftRegistryID;",
                DataAccess.CreateParameterString( giftRegistryID )
            );

            if (table.Rows.Count > 0)
                return CreateOneFromRow( table.Rows[0] );
            else
                return GiftRegistry.Null;
        }

        public void Delete( string giftRegistryID )
        {
            DataAccessContext.GiftRegistryItemRepository.DeleteByGiftRegistryID( giftRegistryID );

            DataAccess.ExecuteNonQuery(
                "DELETE FROM GiftRegistry " +
                "WHERE GiftRegistryID = @GiftRegistryID;",
                DataAccess.CreateParameterString( giftRegistryID )
            );
        }

        public IList<GiftRegistry> GetAllByUserName( string userName, string storeID )
        {
            string customerID = DataAccessContext.CustomerRepository.GetIDFromUserName( userName );
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM GiftRegistry " +
                "WHERE CustomerID = @CustomerID AND StoreID = @StoreID ORDER BY EventDate;",
                DataAccess.CreateParameterString( customerID ),
                DataAccess.CreateParameterString( storeID ) );

            return ConvertTableToList( table );
        }

        public IList<GiftRegistry> Search(
            string firstName,
            string lastName,
            string eventName,
            string startEventDate,
            string endEventDate,
            string storeID,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            ArrayList parameters = new ArrayList();
            parameters.Add( DataAccess.CreateParameterString( storeID ) );
            parameters.Add( DataAccess.CreateParameterBool( false ) );

            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllFields +
                "FROM GiftRegistry " +
                "WHERE StoreID = @StoreID AND HideEvent = @HideEvent " +
                ConditionString( "FirstName", firstName, parameters ) +
                ConditionString( "LastName", lastName, parameters ) +
                ConditionString( "EventName", eventName, parameters ) +
                ConditionDate( startEventDate, endEventDate, parameters ) + ";",
                DataAccess.ConvertArrayListToDbParameters( parameters )
                );

            howManyItems = table.Rows.Count;

            return ConvertTableToList( table, startIndex, endIndex );
        }

        #endregion
    }
}
