using System;
using System.Collections.Generic;
using System.Data.Common;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Orders;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class OrderRepository : IOrderRepository
    {
        #region Private

        private const string RepositoryName = "OrderRepository";

        private const string GetOneSignature = "GetOne";
        private const string SearchOrderSignature = "SearchOrder";
        private const string ExportOrderSignature = "ExportOrder";
        private const string SumOrderSignature = "SumOrder";
        private const string GetOrdersByProcessedAndStatusSignature = "GetOrdersByProcessedAndStatus";
        private const string GetPaidOrdersByDateTimeSignature = "GetPaidOrdersByDateTime";
        private const string GetOrderSignature = "GetOrder";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetTableOrderHistorySchemaSignature = "GetTableOrderHistorySchema";
        private const string GetMaximumOrderIDSignature = "GetMaximumOrderID";
        private const string GetOrderIDByGatewayIDSignature = "GetOrderIDByGatewayID";
        private const string GetLastestOrderSignature = "GetLastestOrder";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "Orders", "OrderNote", "OrderTracking" } );

        private OrderRepositoryNoCache _repository = new OrderRepositoryNoCache();

        #endregion

        #region Constructors

        public OrderRepository()
        {
        }

        public OrderRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Order Save( Order order )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Order newOrder = _repository.Save( order );
            _dataCache.ClearRepositoryCache();
            return newOrder;
        }

        public void Delete( string orderID )
        {
            _repository.Delete( orderID );
            _dataCache.ClearRepositoryCache();
        }

        public Order GetOne( string orderID )
        {
            Order result = (Order) _dataCache.GetValue( GetOneSignature, _cacheDependency, orderID );

            if (result == null)
            {
                result = _repository.GetOne( orderID );
                _dataCache.SetValue( GetOneSignature, result, orderID );
            }

            return result;
        }

        private string ConvertDbParameterArrayToString( DbParameter[] parameterExtraFilter )
        {
            string outputString = string.Empty;
            foreach (DbParameter dbP in parameterExtraFilter)
            {
                outputString += "," + dbP.Value.ToString();
            }
            return outputString;
        }

        public IList<Order> SearchOrder(
            string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            int startIndex,
            int endIndex,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter
           )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchOrderSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                extraFilter,
                storeID,
                startIndex,
                endIndex,
                ConvertDbParameterArrayToString(parameterExtraFilter)
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchOrder(
                    sortBy, searchFilter, extraFilter, storeID, startIndex, endIndex, out howManyItems, parameterExtraFilter );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchOrderSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    extraFilter,
                    storeID,
                    startIndex,
                    endIndex,
                    ConvertDbParameterArrayToString(parameterExtraFilter) );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Order>) result.First;
        }

        public IList<Order> SearchOrder(
            string sortBy,
            SearchFilter searchFilter,
            OrderCreateExtraFilter extraFilter,
            string storeID,
            int startIndex,
            int endIndex,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchOrderSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                extraFilter.ToString(),
                storeID,
                startIndex,
                endIndex,
                ConvertDbParameterArrayToString(parameterExtraFilter)
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchOrder(
                    sortBy, searchFilter, extraFilter, storeID, startIndex, endIndex, out howManyItems, parameterExtraFilter );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchOrderSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    extraFilter.ToString(),
                    storeID,
                    startIndex,
                    endIndex,
                    ConvertDbParameterArrayToString(parameterExtraFilter) );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Order>) result.First;
        }

        public IList<Order> SearchOrder(
           string sortBy,
            SearchFilter searchFilter,
            string extraFilter,
            string storeID,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchOrderSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                extraFilter,
                storeID,
                ConvertDbParameterArrayToString(parameterExtraFilter)
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchOrder(
                    sortBy, searchFilter, extraFilter, storeID, out howManyItems, parameterExtraFilter );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchOrderSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    extraFilter,
                    storeID,
                    ConvertDbParameterArrayToString(parameterExtraFilter) );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Order>) result.First;
        }

        public IList<Order> SearchOrder(
           string sortBy,
            SearchFilter searchFilter,
            OrderCreateExtraFilter extraFilter,
            string storeID,
            out int howManyItems,
            params DbParameter[] parameterExtraFilter )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchOrderSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                extraFilter.ToString(),
                storeID,
                ConvertDbParameterArrayToString(parameterExtraFilter)
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchOrder(
                    sortBy, searchFilter, extraFilter, storeID, out howManyItems, parameterExtraFilter );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchOrderSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    extraFilter.ToString(),
                    storeID,
                    ConvertDbParameterArrayToString(parameterExtraFilter) );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Order>) result.First;
        }

        public IList<Order> ExportOrder(
            string sortBy,
            SearchFilter searchFilter,
            OrderCreateExtraFilter extraFilter,
            string storeID,
            params DbParameter[] parameterExtraFilter
            )
        {
            IList<Order> list = (IList<Order>) _dataCache.GetValue(
                ExportOrderSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                extraFilter.ToString(),
                storeID,
                ConvertDbParameterArrayToString(parameterExtraFilter) );

            if (list == null)
            {
                list = _repository.ExportOrder( sortBy, searchFilter, extraFilter, storeID, parameterExtraFilter );
                _dataCache.SetValue(
                    ExportOrderSignature,
                    sortBy,
                    searchFilter.ToString(),
                    extraFilter.ToString(),
                    storeID,
                    ConvertDbParameterArrayToString(parameterExtraFilter) );
            }

            return list;
        }

        public decimal SumOrder(
            SearchFilter searchFilter,
            OrderCreateExtraFilter extraFilter,
            string storeID )
        {

            decimal result = ConvertUtilities.ToDecimal( _dataCache.GetValue(
                SumOrderSignature,
                _cacheDependency,
                searchFilter.ToString(),
                extraFilter.ToString(),
                storeID
                ) );

            if (result == 0)
            {
                result = _repository.SumOrder( searchFilter, extraFilter,storeID );
                _dataCache.SetValue(
                    SumOrderSignature,
                    result,
                    searchFilter.ToString(),
                    extraFilter.ToString(),
                    storeID
                    );
            }

            return result;
        }

        public IList<Order> GetOrdersByProcessedAndStatus( bool processed, string status )
        {
            IList<Order> result = (IList<Order>) _dataCache.GetValue(
                GetOrdersByProcessedAndStatusSignature,
                _cacheDependency,
                processed,
                status
                );

            if (result == null)
            {
                result = _repository.GetOrdersByProcessedAndStatus( processed, status );
                _dataCache.SetValue(
                    GetOrdersByProcessedAndStatusSignature,
                    result,
                    processed,
                    status
                     );
            }

            return result;
        }

        public IList<Order> GetOrder(
            int startOrderID,
            int numberOfDays,
            int limitCount )
        {
            IList<Order> result = (IList<Order>) _dataCache.GetValue(
                 GetOrderSignature,
                 _cacheDependency,
                 startOrderID,
                 numberOfDays,
                 limitCount
                 );

            if (result == null)
            {
                result = _repository.GetOrder( startOrderID, numberOfDays, limitCount );
                _dataCache.SetValue(
                    GetOrderSignature,
                    result,
                    startOrderID,
                    numberOfDays,
                    limitCount
                     );
            }
            return result;
        }

        public IList<Order> GetPaidOrdersByDateTime( DateTime? from, DateTime? to )
        {
            IList<Order> result = (IList<Order>) _dataCache.GetValue(
                GetPaidOrdersByDateTimeSignature,
                _cacheDependency,
                from.Value.ToString( Culture.USCultureInfo ),
                to.Value.ToString( Culture.USCultureInfo )
                );

            if (result == null)
            {
                result = _repository.GetPaidOrdersByDateTime( from, to );
                _dataCache.SetValue(
                    GetPaidOrdersByDateTimeSignature,
                    result,
                    from.Value.ToString( Culture.USCultureInfo ),
                    to.Value.ToString( Culture.USCultureInfo )
                     );
            }

            return result;
        }

        public IList<Order> GetLastestOrder( int limitCount )
        {
            IList<Order> result = (IList<Order>) _dataCache.GetValue(
                GetLastestOrderSignature,
                _cacheDependency,
                limitCount );
            if (result == null)
            {
                result = _repository.GetLastestOrder( limitCount );
                _dataCache.SetValue(
                    GetLastestOrderSignature,
                    result, limitCount);
            }

            return result;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature,
                   _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableOrderHistorySchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableOrderHistorySchemaSignature,
                   _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableOrderHistorySchema();
                _dataCache.SetValue(
                    GetTableOrderHistorySchemaSignature,
                    list
                    );
            }

            return list;
        }

        public string GetMaximumOrderID()
        {
            string result = (String) _dataCache.GetValue(
                   GetMaximumOrderIDSignature,
                   _cacheDependency
                   );

            if (String.IsNullOrEmpty( result ))
            {
                result = _repository.GetMaximumOrderID();
                _dataCache.SetValue(
                    GetMaximumOrderIDSignature,
                    result
                    );
            }

            return result;
        }


        public void SetStartupOrderID( string startOrderID )
        {
            _repository.SetStartupOrderID( startOrderID );
        }

        public string GetOrderIDByGatewayID( string gatewayOrderID )
        {
            string result = (String) _dataCache.GetValue(
                   GetOrderIDByGatewayIDSignature,
                   _cacheDependency,
                   gatewayOrderID
                   );

            if (String.IsNullOrEmpty( result ))
            {
                result = _repository.GetOrderIDByGatewayID( gatewayOrderID );
                _dataCache.SetValue(
                    GetOrderIDByGatewayIDSignature,
                    result,
                    gatewayOrderID
                    );
            }

            return result;

        }

        public IList<Order> GetByIsQBExported( bool qbIsExported, string storeID, string existList )
        {
            IList<Order> result = _repository.GetByIsQBExported( qbIsExported, storeID, existList );
            return result;
        }

        public void SetDefaultQBData( string qbTxnID )
        {
            _repository.SetDefaultQBData( qbTxnID );
            _dataCache.ClearRepositoryCache();
        }

        #endregion
    }
}
