using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Shipping;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class ShippingOptionRepository : IShippingOptionRepository
    {
        #region Private

        private const string RepositoryName = "ShippingOptionRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetLocalesSignature = "GetLocales";
        private const string SearchShippingSignature = "SearchShipping";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetIDFromNameSignature = "GetIDFromName";
        private const string GetShippingSignature = "GetShipping";
        private const string GetAllByShippingZoneGroupIDSignature = "GetAllByShippingZoneGroupID";
        private const string GetShippingZoneSignature = "GetShippingZone";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "ShippingLocale", "Shipping", "ShippingOptionType", "ShippingZone" } );

        private ShippingOptionRepositoryNoCache _repository = new ShippingOptionRepositoryNoCache();

        #endregion

        #region Constructors

        public ShippingOptionRepository()
        {
        }

        public ShippingOptionRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public ShippingOption Save( ShippingOption shippingOption )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            ShippingOption newShippingOption = _repository.Save( shippingOption );
            _dataCache.ClearRepositoryCache();
            return newShippingOption;
        }

        public void Delete( string shippingID )
        {
            _repository.Delete( shippingID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public ShippingOption GetOne( Culture culture, string shippingID )
        {
            ShippingOption result = (ShippingOption) _dataCache.GetValue(
                GetOneSignature,
                _cacheDependency,
                culture.CultureID,
                shippingID );

            if (result == null)
            {
                result = _repository.GetOne( culture, shippingID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, shippingID );
            }

            return result;
        }

        public IList<ILocale> GetLocales( string shippingID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue(
                GetLocalesSignature,
                _cacheDependency,
                shippingID );

            if (list == null)
            {
                list = _repository.GetLocales( shippingID );
                _dataCache.SetValue( GetLocalesSignature, list, shippingID );
            }

            return list;
        }

        public IList<ShippingOption> SearchShipping(
            Culture culture,
            string sortBy,
            int startIndex,
            int endIndex,
            out int howManyItems )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchShippingSignature,
                _cacheDependency,
                culture.CultureID,
                sortBy,
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchShipping(
                    culture, sortBy, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchShippingSignature,
                    result,
                    culture.CultureID,
                    sortBy,
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<ShippingOption>) result.First;
        }

        public string GetIDFromName( string name )
        {
            string result = ConvertUtilities.ToString( _dataCache.GetValue(
                GetIDFromNameSignature,
                _cacheDependency,
                name
                ) );

            if (result == null ||
                string.IsNullOrEmpty( result ))
            {
                result = _repository.GetIDFromName( name );

                _dataCache.SetValue(
                    GetIDFromNameSignature,
                    result,
                    name
                    );
            }

            return result;
        }

        public IList<ShippingOption> GetShipping( Culture culture, BoolFilter isRealTimeShipping )
        {
            IList<ShippingOption> list = (IList<ShippingOption>) _dataCache.GetValue(
                GetShippingSignature,
                _cacheDependency,
                culture.CultureID,
                isRealTimeShipping.ToString() );

            if (list == null)
            {
                list = _repository.GetShipping( culture, isRealTimeShipping );
                _dataCache.SetValue( GetShippingSignature, list, culture.CultureID, isRealTimeShipping.ToString() );
            }

            return list;
        }


        public IList<ShippingOption> GetAllByShippingZoneGroupID( Culture culture, string shippingZoneGroupID )
        {
            IList<ShippingOption> list = (IList<ShippingOption>) _dataCache.GetValue(
                GetAllByShippingZoneGroupIDSignature,
                _cacheDependency,
                culture.CultureID,
                shippingZoneGroupID );

            if (list == null)
            {
                list = _repository.GetAllByShippingZoneGroupID( culture, shippingZoneGroupID );
                _dataCache.SetValue( GetAllByShippingZoneGroupIDSignature, list, culture.CultureID, shippingZoneGroupID );
            }

            return list;
        }


        public IList<ShippingZone> GetShippingZone( string shippingID )
        {
            IList<ShippingZone> list = (IList<ShippingZone>) _dataCache.GetValue(
                GetShippingZoneSignature, _cacheDependency, shippingID );

            if (list == null)
            {
                list = _repository.GetShippingZone( shippingID );
                _dataCache.SetValue( GetShippingZoneSignature, list, shippingID );
            }

            return list;
        }

        #endregion
    }
}
